import pytest
from datetime import datetime, UTC
from atkins.services.redeem_code import RedeemCodeService, insert_dashes


@pytest.fixture
def redeem_code_service(db):
    service = RedeemCodeService(db)
    service.create_collections()
    service.build_index()
    return service


def test_insert_dashes():
    assert insert_dashes("12345678") == "1234-5678"
    assert insert_dashes("123456789") == "1234-5678-9"
    assert insert_dashes("") == ""


def test_generate_secure_code():
    code = RedeemCodeService.generate_secure_code()
    assert len(code.replace("-", "")) == 32  # 16 bytes = 32 hex chars
    assert "-" in code
    assert len(code.split("-")) == 8  # 32 chars / 4 = 8 segments


def test_create_code(redeem_code_service):
    code = "TEST-1234-5678"
    
    # Create code
    created_code = redeem_code_service.create_code(code)
    assert created_code == code
    
    # Verify code exists
    code_record = redeem_code_service.get_code(code)
    assert code_record is not None
    assert code_record["code"] == code
    assert code_record["used"] is False
    assert code_record["status"] == "issued"
    assert isinstance(code_record["createdAt"], datetime)
    assert code_record["request"] is None
    assert code_record["response"] is None
    assert code_record["error"] is None


def test_start_processing_code(redeem_code_service):
    code = "TEST-1234-5678"
    request = {"type": "test", "value": 100}
    
    # Create code
    redeem_code_service.create_code(code)
    
    # Start processing
    assert redeem_code_service.start_processing_code(code, request) is True
    
    # Verify processing state
    code_record = redeem_code_service.get_code(code)
    assert code_record["status"] == "processing"
    assert code_record["request"] == request
    assert isinstance(code_record["updatedAt"], datetime)


def test_mark_processing_failed(redeem_code_service):
    code = "TEST-1234-5678"
    error_msg = {"message": "Test error", "code": "TEST_ERROR"}
    
    # Create code and start processing
    redeem_code_service.create_code(code)
    redeem_code_service.start_processing_code(code, {"type": "test"})
    
    # Mark as failed
    assert redeem_code_service.mark_processing_failed(code, error_msg) is True
    
    # Verify failed state
    code_record = redeem_code_service.get_code(code)
    assert code_record["status"] == "error"
    assert code_record["error"] == error_msg
    assert isinstance(code_record["updatedAt"], datetime)


def test_mark_processing_done(redeem_code_service):
    code = "TEST-1234-5678"
    response = {"success": True, "message": "Test completed"}
    
    # Create code and start processing
    redeem_code_service.create_code(code)
    redeem_code_service.start_processing_code(code, {"type": "test"})
    
    # Mark as done
    assert redeem_code_service.mark_processing_done(code, response) is True
    
    # Verify completed state
    code_record = redeem_code_service.get_code(code)
    assert code_record["status"] == "used"
    assert code_record["response"] == response
    assert isinstance(code_record["updatedAt"], datetime)


def test_get_nonexistent_code(redeem_code_service):
    code = "NONEXISTENT-CODE"
    code_record = redeem_code_service.get_code(code)
    assert code_record is None


def test_unique_code_constraint(redeem_code_service):
    code = "TEST-1234-5678"
    
    # Create code
    redeem_code_service.create_code(code)
    
    # Try to create same code again
    with pytest.raises(Exception):  # Should raise duplicate key error
        redeem_code_service.create_code(code) 